---
# theme: white
# center: false
slideNumber: true
# controls: false
---

# Speaker Notes

Press s to open speaker notes. The sole presentation was not intended to be self contained.

# Rolling Shutter Camera Synchronization with Sub-millisecond Accuracy

Matěj Šmíd, Jiří Matas \
Center for Machine Perception \
Czech Technical University Prague \
smidm@cmp.felk.cvut.cz

![1 flap of wings takes 5 ms.](figures/bee.gif "Honeybee in slow motion.")
 

<div class="notes">
- def: 
    - independent videos
    - associate time-point in one video to time-points in other videos
- method requires: rolling shutter and a global lighting change
- Honeybee in slow motion. Our accuracy 1/5 to 1/10 of one wings flap.
- slides with speaker notes available
</div>
   
# Multiple Cameras

- are common 
- synchronization is required
- are the only solution for some applications

![Video effect based on interpolation between multiple synchronized cameras.](figures/bullet_time.gif "The Matrix movie bullet-time effect.")

<div class="notes">
- motivation for multi-camera
    - 3D reconstruction
    - human pose estimation
    - tracking
    - occlusion handling
    - multi directional camera rigs
- hw solution for sync not always feasible
</div>

# Tracking Application

<video data-autoplay src="figures/video_floorball.webm" />

<div class="notes">
- our motivation: multi target tracking
</div>

# Ice Hockey Dataset

<video data-autoplay src="figures/merged_1234.webm" />

<div class="notes">
- synchronization is essential
    - information fusion
- IP cameras without hardware trigger
- independently acquired videos
- dataset:
    - ice hockey sequence
    - 4 cameras
    - 5 minutes
</div>

# The Core Idea

synchronize using lighting changes affecting the whole scene

![A flash during a rolling shutter sensor exposure.](figures/artefact.jpg "Photographic flash artefact in a rolling shutter image sensor.")

image overlap not required (!)

<div class="notes">
- horizontal bright stripe covering approximately 1/3 of an image
- professional photographers flashes
- common in indoor sports and cultural events (concerts)
- easy to produce in experimental setting (event outdoor)
</div>

# Detected Events

![Lighting events detected in 4 video sequences.](figures/events_not_synchronized_1_2_3_4.png "Lighting events detected in 4 video sequences.")

<div class="notes">
- detection
    - line wise median
    - difference between consecutive images
- frames with lighting changes detected
</div>


# Synchronization by Mapping Frames

- choose reference camera
- synchronize by adding $n$ whole frames
- assumption: same fps
- $f_\mathrm{ref} = f + n$
    - $f_\mathrm{ref}$ reference camera frame nr.
    - $f$ second camera frame nr.
    
![](figures/frame_shift.svg "Simple synchronization by shifting n frames.") 

<div class="notes">
- synchronization transformation (arrows)
- explain figure
- common in synchronization literature (!)
- different fps (fix by adding fps ratio)
</div>

# Doesn't work {data-background="figures/unloading.gif"}

# Frame Drops

- common phenomena
- often ignored
- cause: high load, lost packets
- synchronization error cumulative

![](figures/frame_shift_fail.svg "Synchronization in domain of frame indices fails in presence of dropped frames.") 

<div class="notes">
- example error: 25 fps, 3 dropped frames:
    - ice hockey player travelled 1 m
    - puck more than 4 m
- TODO: frame drops statistics in my videos
</div>

# Video Timing

```
$ ffprobe -select_streams v
    -show_entries frame=
    best_effort_timestamp_time
    video.mp4
  
frame,0.000000
frame,0.040000
frame,0.080000
frame,0.120000
frame,0.160000
frame,0.200000
frame,0.240000
frame,0.280000
frame,0.320000
frame,0.360000
frame,0.400000
frame,0.440000
```

- use frame timestamps instead of indices
- available:
    - video containers
    - streaming protocols
- why is it ignored?

<div class="notes">
- all synchronization literature operates on frame indices
- ffprobe from ffmpeg package
- do not rely on opencv video timing queries
</div>

# Timestamp based Sync

- fps invariant, frame drops resistant
- $t_\mathrm{ref} = t + \Delta t$
    - $t$ frame timestamp
    - $\Delta t$ time shift

![](figures/time_shift.svg "Synchronization using timestamps. The fps can be arbitrary.") 

<div class="notes">
- new parts of synchronization transformation are in red
- different fps (25, 12.5)
- dropped frame in red camera at 120 ms
</div>

# Rolling Shutter

- frame rows not exposed simultaneously
- small delay between consecutive rows
- disadvantage: image distortion
- advantage: high-speed optical line sensor

![](figures/rolling_shutter.svg "Rolling shutter equipped image sensor integrates light into image rows sequentially.") 

<div class="notes">
- we can exploit rolling shutter properties for accurate synchronization
- almost all sensors sold
- sampling rate: $\mathrm{fps} \cdot H$, where H is vertical resolution
- theoretical sampling time in 10ths of microseconds
</div>

---

# Distortion

![Objects on top of the image and on the bottom are captured in different time instants.](figures/Wien-Prag_rolling_shutter_P1280392.jpg "Rolling shutter image distortion (image courtesy of Ermell).") 

<div class="notes">
- camera moving in a train
- common when moving camera or objects move horizontally
</div>

---

# Lighting Change

![](figures/profile.svg.png "Vertical profile of a row wise median in moment of abrupt lighting change.") 

- e.g. flash, room light switch
- profile: line-wise median, difference between consecutive images

<div class="notes">
- easy to produce
- light change analysis: 
    1. reduction of 2D image into 1D: mean / median over rows
    2. difference between consecutive frames
- shape formed by physical properties of a light source
- fit exp
</div>

<!---

# Change in Detail

- exponential shape
-->

# Going Sub-frame

- we can associate time of a row in arbitrary camera to a time of a row in the reference camera
- $t'_\mathrm{ref} = t + r  T_\mathrm{row} + \Delta t$
    - $r$ row number
    - $T_\mathrm{row}$ time per row

![](figures/subframe_shift.svg "Synchronization using timestamps. The fps can be arbitrary.") 

<div class="notes">
- we can associate time of a row in arbitrary camera to a time of a row in reference camera
</div>

# Doesn't work precisely {data-background="figures/bikefail.gif"}

# Clock Skew

- millisecond scale + image sensor clock generator inaccuracy
- compensation needed ($\beta$)
- $t''_\mathrm{ref} = \beta t + r T_\mathrm{row} + \Delta t$
    - $\beta$ relative clock skew to the reference camera

# Sync Parameters

- lighting event $(t, r)$ frame time, row pair
- pair of corresponding events in 2 cameras = 1 equation
- system of linear equations
- least squares solution

![](figures/events_not_synchronized_1_2_3_4.png "Detected abrupt lighting events in 4 video streams.")
![](figures/events_1_2_3_4.png "Detected abrupt lighting events in 4 video streams.")

<div class="notes">
- extract row and frame time pairs for all detected events
- manually or automatically match the events
    - not all events need to be matched
- construct system of linear equations
- solve to get time shifts, camera skew and sub-frame constants
</div>

# Results

<video data-autoplay src="figures/homography.webm" />

<div class="notes">
- every colour represent moving objects from one camera
- merged using ground plane homography -> points on the ground are aligned
</div>

# Results

![](figures/results.png "Results table.")

- residual errors < 1ms
- ice hockey dataset, 5 minutes, 4 cameras

# Open Source

```python
import flashvideosynchronization
sync = flashvideosynchronization.FlashVideoSynchronization()
sync.detect_flash_events(filenames)

matching_events = {1: 3, 3: 2, 2: 8, 4: 2}
offsets = {
    cam: sync.events[cam][matching_events[cam]]['time'] 
    for cam in cameras}

# synchronize cameras: find parameters of transformations 
# that map camera time to reference camera time
sync.synchronize(cameras, offsets, base_cam=1)

# get sub-frame sychronized time for camera 1, frame 10 and row 100
print sync.get_time(cam=1, frame_time=timestamps[1][10], row=100) 
```

https://github.com/smidm/flashvideosynchronization

<div class="notes">
- python module
- easy to use
</div>

# Summary

- sub-millisecond accuracy for rolling shutter cameras
- requirement: global detectable lighting changes
- open source

http://cmp.felk.cvut.cz/~smidm/flash_synchronization

<div class="notes">
</div>

<!---

Please use 15 minutes for presenting your paper and leave 5 minutes for the question-answering period.

“All technical session rooms are equipped with a video projector and a computer running Microsoft Windows with Microsoft PowerPoint and Adobe Acrobat Reader. It is recommended that your presentation is uploaded to PRIMORIS before the conference, which ensures that it will be stored at the aforementioned computer by the time it is needed. However, you may prefer to bring it in a USB drive (also recommended as a backup precaution). Using your own laptop or smart device, as long as it is enabled with VGA output connection is possible but not recommended. The presenter should prepare a set of slides that clearly illustrate the main points of her/his paper, and make sure the presentation fits well within the assigned time slot, leaving at least 5 minutes for a period of question-answering. Should sound be required, the presenter is kindly requested to contact the conference secretariat at least 3 weeks in advance in order to ensure proper arrangements. The rooms are not normally equipped with a sound amplification system.”


ffprobe -hide_banner -select_streams v -show_entries frame=best_effort_timestamp_time -of csv 1.mp4  -noprivate 
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '1.mp4':
  Metadata:
    major_brand     : isom
    minor_version   : 512
    compatible_brands: isomiso2avc1mp41
    encoder         : Lavf56.31.100
  Duration: 00:31:15.80, start: 0.000000, bitrate: 53058 kb/s
    Stream #0:0(eng): Video: h264 (High) (avc1 / 0x31637661), yuv420p(tv, bt709), 3840x2160 [SAR 1:1 DAR 16:9], 53056 kb/s, 24.86 fps, 25 tbr, 16k tbn, 32k tbc (default)
    Metadata:
      handler_name    : VideoHandler
frame,0.000000
frame,0.040000
frame,0.080000
frame,0.120000
frame,0.160000
frame,0.200000
frame,0.240000
frame,0.280000
frame,0.320000
frame,0.360000
frame,0.400000
frame,0.440000
frame,0.480000
frame,0.520000
frame,0.560000


# get frame synchronized image sources
sources_sync = sync.get_synchronized_image_sources(sources, master=1, dropped=False)

# show synchronized frames
frame = 10
for cam in cameras:
    plt.figure()
    plt.imshow(sources_sync[cam].get_image(frame))
-->
